@@ -199,8 +199,17 @@ class LinuxDevice(object):
         if self.attributes.get("autologin", True):
             self.psession = self.login()
             self.connected = True
-            self.name = LinuxDevice.cmd(self, "hostname", log_level = self.LOG_CRIT).strip()
 
+            # It's a bug workaround for the issues/669:
+            # https://github.com/pexpect/pexpect/issues/669
+            LinuxDevice.cmd(self,
+                'echo "set enable-bracketed-paste off" > .workaround_for_pexpect_issue669',
+                log_level = self.LOG_CRIT)
+            LinuxDevice.cmd(self,
+                    'export INPUTRC=$PWD/.workaround_for_pexpect_issue669',
+                    log_level = self.LOG_CRIT)
+
+            self.name = LinuxDevice.cmd(self, "hostname", log_level = self.LOG_CRIT).strip()
             if self.name:
                 self["name"] = self["name"] and self["name"] or self.name.split('.')[0]
             else:
@@ -450,21 +459,15 @@ class LinuxDevice(object):
         len2 = self.psession.before.decode("utf-8").rfind("\r\n")
         output = self.psession.before[len1:len2].decode("utf-8").strip("\r\n")
 
-        # Some devices print characters after the prompt. We will eat those
-        # characters and print them in log, but will not copy it to the output.
-        self.psession.sendline("")
-        i = self.psession.expect(prompt, 0.1)
-        output2 = (i>=2) and self.psession.before.decode("utf-8").strip("\r\n") or None
-
         if kwargs.get("redirect", False):
             # If redirect option is set then don't log to stdout
             self.log("redirect the output >> %s ...\n" %(kwargs["redirect"]))
             redirect_file = open(kwargs["redirect"], "a+")
-            redirect_file.write((output2 and (output + output2) or output) + "\r\n\r\n")
+            redirect_file.write(output + "\r\n\r\n")
             redirect_file.flush()
             redirect_file.close()
         else:
-            self.log("%s" %(output2 and (output + output2) or output),
+            self.log("%s" %(output),
                     level = LOG_INFO,
                     leading = "  ",
                     #color = output_color,
