@@ -2,7 +2,6 @@
 
 from netdevice import linux
 import simplejson as json
-import demjson
 import pexpect
 import xmltodict
 import ipaddress
@@ -39,19 +38,19 @@ class OvsHost(linux.LinuxDevice):
             self.cmd('ovs-vsctl --no-wait set Open_vSwitch . other_config:dpdk-init=true')
             self.log("Setup OVS complete")
 
-        vlog = self["vlog"] and int(self["vlog"]) or -1
-        if (vlog >= 0):
+        self["vlog"] = self["vlog"] and int(self["vlog"]) or -1
+        if (self["vlog"] >= 0):
             self.cmd('echo > /usr/local/var/log/openvswitch/ovs-vswitchd.log')
             #self.cmd('echo > /usr/local/var/log/openvswitch/ovsdb-server.log')
-            if (vlog == 0):
+            if (self["vlog"] == 0):
                 self.cmd('ovs-appctl vlog/set ANY:file:emer')
-            elif (vlog == 1):
+            elif (self["vlog"] == 1):
                 self.cmd('ovs-appctl vlog/set ANY:file:err')
-            elif (vlog == 2):
+            elif (self["vlog"] == 2):
                 self.cmd('ovs-appctl vlog/set ANY:file:warn')
-            elif (vlog == 3):
+            elif (self["vlog"] == 3):
                 self.cmd('ovs-appctl vlog/set ANY:file:info')
-            elif (vlog >= 4):
+            elif (self["vlog"] >= 4):
                 self.cmd('ovs-appctl vlog/set ANY:file:dbg')
 
         # start the docker if it's not running
@@ -122,9 +121,9 @@ class OvsHost(linux.LinuxDevice):
             self.add_port(bridge_name, **p)
 
         # Add remote-device and it's peer self-port
-        for d in args:
-            print("d: " %(d))
-            self.add_device(bridge_name, **d)
+        #for d in args:
+        #    print("d: " %(d))
+        #    self.add_device(bridge_name, **d)
 
     def add_port (self, bridge_name, name, **kwargs):
         '''
@@ -138,48 +137,105 @@ class OvsHost(linux.LinuxDevice):
                 command += ' %s=%s' %(k,v)
         self.cmd(command) #execut the command.
 
-    def add_device (self, bridge_name, name, *args, **kwargs):
+    def add_vm (self, mask = None,
+            type = "qemu-system-x86_64",
+            name = "vm1",
+            cpu = "host", 
+            smp = "sockets=1,cores=1",
+            m = "1024M", 
+            hda = None,
+            boot = "c",
+            net = "none",
+            pidfile = "/tmp/vm1.pid",
+            monitor = "unix:/tmp/vm1monitor,server,nowait",
+            object = "memory-backend-file,id=mem,size=1024M,mem-path=/dev/hugepages,share=on",
+            numa = "node,memdev=mem",
+            chardev = None,
+            netdev = None,
+            device = None,
+            **kwargs):
+        '''
+        taskset 8 qemu-system-x86_64 -name $VM_NAME -cpu host -smp sockets=1,cores=1 -m 1024M -hda $QCOW2_IMAGE -boot c -enable-kvm -net none -no-reboot -pidfile /tmp/vm1.pid -monitor unix:/tmp/vm1monitor,server,nowait -object memory-backend-file,id=mem,size=1024M,mem-path=/dev/hugepages,share=on -numa node,memdev=mem -mem-prealloc -chardev socket,id=char1,path=$VHOST_SERVER_PATH,server -netdev type=vhost-user,id=net1,chardev=char1,vhostforce -device virtio-net-pci,netdev=net1,mac=00:00:00:00:00:01,csum=off,gso=off,guest_tso4=off,guest_tso6=off,guest_ecn=off,mrg_rxbuf=off
+        '''
+        cmd = mask and ("taskset %s %s" %(mask, type)) or type
+        cmd += " -name %s -cpu %s -smp %s -m %s -hda %s -boot %s"\
+                %(name, cpu, smp, m, hda, boot)
+        if kwargs.get("enable-kvm", True):
+            cmd += " -enable-kvm"
+        if kwargs.get("no-reboot", True):
+            cmd += " -no-reboot"
+        if kwargs.get("mem-prealloc", True):
+            cmd += " -mem-prealloc"
+        cmd += " -net %s -pidfile %s -monitor %s -object %s -numa %s"\
+                %(net, pidfile, monitor, object, numa)
+
+        if (device):
+            # if the chardev/netdev/device are given, use them
+            chardev = isinstance(chardev, list) and chardev or [chardev]
+            netdev = isinstance(netdev, list) and netdev or [netdev]
+            device = isinstance(device, list) and device or [device]
+            all_chardev = {}
+            all_netdev = {}
+            for c in chardev:
+                # Parse the chardev to find its type.
+                tmp_dic = {}
+                for a in c.split(","):
+                    k,v = ("=" in a) and a.split("=") or (a, True)
+                    tmp_dic[k.strip()] = (isinstance(v, str)) and v.strip() or v
+                all_chardev[tmp_dic["id"]] = "-chardev %s" %(c)
+            #print(all_chardev)
+            for n in netdev:
+                # Parse the netdev to find its type.
+                tmp_dic = {}
+                for a in n.split(","):
+                    k,v = ("=" in a) and a.split("=") or (a, True)
+                    tmp_dic[k.strip()] = (isinstance(v, str)) and v.strip() or v
+                if ("chardev" in tmp_dic):
+                    all_netdev[tmp_dic["id"]] = " %s -netdev %s"\
+                            %(all_chardev[tmp_dic["chardev"]], n)
+                else:
+                    all_netdev[tmp_dic["id"]] = "-netdev %s" %(n)
+            #print(all_netdev)
+            for d in device:
+                # Parse the device to find its type.
+                tmp_dic = {}
+                for a in d.split(","):
+                    k,v = ("=" in a) and a.split("=") or (a, True)
+                    tmp_dic[k.strip()] = (isinstance(v, str)) and v.strip() or v
+                cmd += " %s -device %s" %(all_netdev[tmp_dic["netdev"]], d)
+        print(cmd)
+        #val = os.system(cmd + "&")
+        #self.cmd("whoami")
+        #self.cmd(cmd)
+
+    def add_devices (self, bridge_name, *args, **kwargs):
         '''
         connect devices to ovs, now only support container.
         '''
         # Add remote-device and it's peer self-port
-        peer_type = kwargs.get("type", None)
-        if peer_type == "container":
-            # Create a container and the self-port
-            self.cmd('docker run --privileged -d --name %s -v %s:%s -it centos'
-                    %(name, kwargs.get("host_dir", "/var/shared"),
-                      kwargs.get("container_dir", "/var/shared")))
+        for d in args:
+            dtype = d.get("type", None)
+            if dtype == "docker":
+                # Create a container and the self-port
+                ports = d.pop("port", []) # get the ports list
+                for p in ports:
+                    self.cmd('docker run --privileged -d --name %s -v %s:%s -it centos'
+                            %(p["name"], p.get("host_dir", "/var/shared"),
+                              p.get("container_dir", "/var/shared")))
 
-            #Allocate the interface/ipaddress and connect it to ovs bridge
-            self.cmd('ovs-docker add-port %s %s %s --ipaddress=%s'
-                    %(bridge_name, kwargs["interface"], name, kwargs["ip"]))
-        elif peer_type == "kvm":
-            # Create ifup/ifdown script
-            self.cmd('echo "#!/bin/sh" > /etc/ovs-ipup')
-            self.cmd('echo "" > /etc/ovs-ipup')
-            self.cmd('echo "switch=\'%s\'" > /etc/ovs-ipup' %(bridge_name))
-            self.cmd('echo "ip link set $1 up" > /etc/ovs-ipup')
-            self.cmd('echo "ovs-vsctl add-port ${switch} $1" > /etc/ovs-ipup')
-            self.cmd('echo "EOF" > /etc/ovs-ipup')
+                    #Allocate the interface/ipaddress and connect it to ovs bridge
+                    self.cmd('ovs-docker add-port %s %s %s --ipaddress=%s'
+                            %(bridge_name, p["interface"], p["name"], p["ip"]))
+            elif dtype == "qemu-system-x86_64":
+                # Create ifup/ifdown script
+                self.add_vm(**d)
+            else:
+                self.log("device type %s is not supported now..." %(dtype))
 
-            self.cmd('echo "#!/bin/sh" > /etc/ovs-ipdown')
-            self.cmd('echo "" > /etc/ovs-ipdown')
-            self.cmd('echo "switch=\'%s\'" > /etc/ovs-ipdown' %(bridge_name))
-            self.cmd('echo "ip addr flush dev $1" > /etc/ovs-ipdown')
-            self.cmd('echo "ip link set $1 down" > /etc/ovs-ipdown')
-            self.cmd('echo "ovs-vsctl del-port ${switch} $1" > /etc/ovs-ipdown')
-            self.cmd('echo "EOF" > /etc/ovs-ipdown')
-
-            self.cmd('kvm -m 512 -net nic,macaddr=00:11:22:EE:EE:EE -net \
-                    tap,script=/etc/ovs-ifup,downscript=/etc/ovs-ifdown -drive \
-                    file=/path/to/disk-image,boot=on')
-        else:
-            self.log("device type %s is not supported now..." %(peer_type))
-
-        # Configure vlan on self-port if the device has vlan configured.
-        if kwargs.get("vlan", None):
-            self.cmd('ovs-vsctl set port %s tag=%d'
-                    %(__ovs_get_port(bridge_name, name = name), kwargs["vlan"]))
+            # Configure vlan on self-port if the device has vlan configured.
+            #if d.get("vlan", None):
+            #    self.cmd('ovs-vsctl set port %s tag=%d'
+            #            %(__ovs_get_port(bridge_name, name = name), d["vlan"]))
 
     def del_br (self, *args, **kwargs):
         '''
